!===============================================================================
!		CUSTOM PROGRAM TO GET RID OF SPURIOUS SALT IN CARDAMOM MODEL
!===============================================================================

program CARDAMOM_SALT_CLEANUP
	use sep
	implicit none

	integer								:: nrbf,nz,nx,ny,nz2,nx2,ny2,ix1,ix2,iy1,iy2
	real,dimension(:,:,:),allocatable	:: input, output
	logical								:: verbose
	integer,dimension(8)               	:: timer0,timer1
	real								:: timer_sum=0., replace_val, salt_val

	call sep_init()
	call DATE_AND_TIME(values=timer0)

	call from_param("verbose",verbose,.TRUE.)
	if(verbose) write(0,*) "==================== Read in initial parameters"
	call from_history("n1",nz)
	call from_history("n2",nx)
	call from_history("n3",ny)
	call from_param("ix1",ix1,1)
	call from_param("ix2",ix2,nx)
	call from_param("iy1",iy1,1)
	call from_param("iy2",iy2,ny)

	IF(verbose) WRITE(0,*) "========= Allocate ==================================="
	allocate(input(nz,nx,ny))
	allocate(output(nz,nx,ny))

	replace_val = MINVAL(input)
	salt_val = MAXVAL(input)

	IF(verbose) WRITE(0,*) "========= Read input =================="
	CALL sep_read(input)

!##########################################################################################
	output = input
	output(:,:,1:ix1)= replace_val
	output(:,:,ix2:nx)= replace_val
	output(:,1:iy1,:)= replace_val
	output(:,iy2:ny,:)= replace_val

	CALL sep_write(output)

!##########################################################################################
	deallocate(input,output)

	!-----------------------------------------------------------------------------
	call DATE_AND_TIME(values=timer1)
	timer0 = timer1-timer0
	timer_sum = timer_sum + (((timer0(3)*24+timer0(5))*60+timer0(6))*60+timer0(7))*1000+timer0(8)
	write(0,*) "timer sum (sec)",timer_sum/1000
	write(0,*) "timer sum (min)",timer_sum/1000/60
	write(0,*) "DONE!"

end program