

#XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
#
#   PROGRAM GRABS SHOT GATHERS AND DOES PROCESSING ON THEM
#
#XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX


#!/usr/local/bin/python3.5
import sys
import os
import math
import commands
import sepbase
import sep2npy
import numpy as np
import subprocess


#=============================================================
class shotgather_matching():
	# Initialize the common variables
	def __init__(self, obsGrid, synGrid, foldmap, writepath, pbsTemplate, binPath, outputPrefix):
		self.obsGrid = obsGrid
		self.synGrid = synGrid
		self.foldmap = foldmap
		self.writepath = writepath
		self.pbsTemplate = pbsTemplate
		self.binPath = binPath
		self.outputPrefix = outputPrefix
		return

	# Get grid key info
	def get_grid_key(self,header,key):
		cmd1 = 'Get parform=n gff < %s' % (header)
		junk,gffFile=commands.getstatusoutput(cmd1)
		cmd2 = 'Get parform=n %s < %s' % (key,gffFile)
		junk,keyVal = commands.getstatusoutput(cmd2)
		return keyVal

	# Check grid cell function to see if it is empty
	def check_grid_cell(self,ix,iy):
		cmd = 'Window3d n1=1 n2=1 f1=%d f2=%d < %s | Attr want=rms param=1 | Get parform=n rms' % (ix,iy,self.foldmap)
		junk,cellRMS = commands.getstatusoutput(cmd)
		return cellRMS

	# Write the executables for shotgather matching	
	def write_shot_executable(self,ix,iy,synFileName,obsFileName,outputFileName,offsetFilename,PBSscriptName):
		copy="cp %s %s" % (self.pbsTemplate,PBSscriptName)
		winObs="Window3d synch=1 n3=1 n4=1 f3=%d f4=%d gff=%s@@@@ hff=%s@@ < %s > %s" % (ix,iy,obsFileName,obsFileName,self.obsGrid,obsFileName)
		winSyn="Window3d synch=1 n3=1 n4=1 f3=%d f4=%d gff=%s@@@@ hff=%s@@ < %s > %s" % (ix,iy,synFileName,synFileName,self.synGrid,synFileName)
		winOff="Window3d synch=1 n1=1 f1=7 < %s@@ > %s" % (synFileName,offsetFilename)
		mute="Mute "
		match="%s/Match_data_win.x offset=%s training=%s convolve=%s nw=300 vel=1700.0 t0=0.25 niter=25 nlag=150 < %s > %s" \
		% (offsetFilename,synFileName,obsFileName,obsFileName,self.binPath)
		writeIt="echo '%s; %s; %s; %s; %s;\n\n' >> %s" % (copy,winObs,winSyn,mute,match,PBSscriptName)
		print(writeIt)
		val,junk = commands.getstatusoutput(writeIt)
		return val

	def submit_PBS_job(PBSscript):
        submit="qsub -l nodes=1:ppn=1 -d %s %s;" % (self.writepath,PBSscript)
		val,junk = commands.getstatusoutput(submit)
        return val



#=============================================================
# MAIN PROGRAM
if __name__ == '__main__':

	# Read the inputs
	if len(sys.argv) < 8:
	    print('Error: Not enough input args')
	    sys.exit()
	obsGrid = sys.argv[1]
	synGrid = sys.argv[2]
	foldmap = sys.argv[3]
	writepath = sys.argv[4]
	pbsTemplate = sys.argv[5]
	binPath = sys.argv[6]
	max_shots_in_job = int(sys.argv[7])
	outputPrefix = sys.argv[8]

	# Initialize function class
	SGmatch=shotgather_matching(obsGrid,synGrid,foldmap,writepath,pbsTemplate,binPath,outputPrefix)

	# Get grid size parameters
	nx=int(SGmatch.get_grid_key(obsGrid,'n3_grid'))
	ny=int(SGmatch.get_grid_key(obsGrid,'n4_grid'))

	# Write the shotgather operations to PBS scripts and launch them
	pbsCount=1
	shots_in_job=0
	maxNumTries=20
	backOffTime=3
	for ix in range(1,nx):
		for iy in range(1,ny):
			shotNum=iy+(ix-1)*nx
			# print(SGmatch.check_grid_cell(ix,iy))
			if (int(SGmatch.check_grid_cell(ix,iy))>0.0):
				# print(shotNum)
				if (shots_in_job<max_shots_in_job):
					synFileName = "%s/synShotGather-%s.H" % (writepath,shotNum)
					obsFileName = "%s/obsShotGather-%s.H" % (writepath,shotNum)
					offsetFileName = "%s/offsetShotGather-%s.H" % (writepath,shotNum)
					outputFileName = "%s/%s-%s.H" % (writepath,outputPrefix,shotNum)
					PBSscriptName = "%s/PBSscript-%s.sh" % (writepath,pbsCount)
					SGmatch.write_shot_executable(ix,iy,synFileName,obsFileName,outputFileName,offsetFilename,PBSscriptName)
					shots_in_job=shots_in_job+1
				else:
					numTries=0
					while (numTries<maxNumTries):
						status=SGmatch.submit_PBS_job(PBSscriptName)
						if (status==1):
							pbsCount=pbsCount+1
							shots_in_job=0
							break
						sleep(backOffTime)
						numTries=numTries+1
						if (numTries==maxNumTries):
							print("Tried %d times to submit job and it kept failing. Moving on to the next job.")
							breakP



	# cmd1="less %s | wc -l" % (recfile)
	# stat1,numNodes=commands.getstatusoutput(cmd1)
	# recList = list(open(recfile))


	# # Visit each location to grab the binned shot data
	# for ii in range(1,int(numNodes)):
	# # for ii in range(0,10):

	#     print("----------------------------------------")
	#     percent=100.0*ii/float(numNodes)
	#     percentcmd="percent done:  %f " % (percent)
	#     print(percentcmd)

	#     nodeId=int(str(recList[ii]).split()[0])
	#     outputFileName="%s/modifiedOBS_%d.H" % (writepath,nodeId)
	#     temp2="%s/temp2_%d.sh" % (writepath,nodeId)
	#     finalPBS="%s/PBSscript_%d.sh" % (writepath,nodeId)

	#     cmdp1="echo '#PBS -o %s/logfile-%s.txt' > %s" % (writepath,nodeId,temp2)
	#     cmdp2="echo '#PBS -e %s/logfile-%s.txt' >> %s" % (writepath,nodeId,temp2)
	#     cmdp3="echo '#PBS -N pbsjob-%s' >> %s" % (nodeId,temp2)
	#     cmdp="%s; %s; %s;" % (cmdp1, cmdp2, cmdp3)
	#     print(cmdp)
	#     subprocess.call(cmdp,shell=True)


	#     # Run the job for a single node
	#     cmd="python %s/nodegather_work_SINGLE.py %d %s %s %s %s %s" % (pythonpath,nodeId,syn_prefix,obs_prefix,writepath,outputFileName,path_out)
	#     cmdW1="echo '%s' >> %s;" % (cmd,temp2)
	#     print(cmdW1)
	#     subprocess.call(cmdW1,shell=True)

	#     cmdW3="cat %s %s > %s;" % (pbsTemplate,temp2,finalPBS)
	#     print(cmdW3)
	#     subprocess.call(cmdW3,shell=True)
	#     cmdW4="qsub -l nodes=1:ppn=24 -d /data/biondo/tjdahlke/research/3dtest/outputs/ %s;" % (finalPBS)
	#     print(cmdW4)
	#     subprocess.call(cmdW4,shell=True)





