#!/usr/bin/python

import time
import subprocess
import math
import copy
from batch_task_executor import *


class BORN(BatchTaskComposer):
    def __init__(self, param_reader, vel_path, prefix, adjointin, modelpath, dataname, debug):
        BatchTaskComposer.__init__(self)

        #Read in the job parameters
        self.dict_args = param_reader.dict_args
        self.T_BIN = self.dict_args['T_BIN']
        self.wavelet = self.dict_args['wavelet']
        self.nshots = int(self.dict_args['nfiles'])
        self.nsh_perjob = int(self.dict_args['nfiles_perjob'])
        self.ish_beg = int(self.dict_args['ish_beg'])
        self.path_out = param_reader.path_out
        self.prefix = prefix
        self.vel_path = vel_path
        self.adjoint = int(adjointin)
        self.dataname = dataname
        self.modelpath = modelpath
        self.genpar = self.dict_args['genpar']
        self.rand_path = self.dict_args['rand_path']

        self.singularity = self.dict_args['singularity']
        self.souhead = self.dict_args['souhead']
        self.recpos = self.dict_args['recpos']

        # Create all subjobs info
        self.njobs = int(math.ceil(self.nshots/float(self.nsh_perjob)))
        self.subjobids = range(0, self.njobs)
        self.subjobfns_list = [None]*self.njobs
        self.subjobIND_list = range(self.ish_beg, self.nshots+1, self.nsh_perjob) # List of actual shot number (starting @ 0)

        # Make a list of all the files for each job ( shots per job)
        for i in range(0,self.njobs):
            ish_start = self.subjobIND_list[i]
            nsh = min(self.nsh_perjob, self.nshots-ish_start)
            self.subjobfns_list[i] = [None]*nsh
            for j in range(0,nsh):
                self.subjobfns_list[i][j] = '%s/%s-%s.H' % (self.path_out,self.prefix,ish_start+j+1)
        return


    def GetSubjobScripts(self, subjobid):
        #'''Return the scripts content for that subjob.'''
        subjobfns = self.subjobfns_list[subjobid]

        # Build scripts that create the files designated in subjobfns
        scripts = []
        cnt = 0
        nf_act = len(subjobfns) # number of files

        # Read in the node position list (since we propagate from the nodes)
        recpos=[]
        tmp=[]
        with open(self.recpos) as f:
            for i, line in enumerate(f):
                line = line.strip()
                tmp=line.split()
                recpos.append(tmp)

        for fn in subjobfns:
            shot = self.nsh_perjob*subjobid+cnt+1
            cnt += 1
            if cnt > nf_act: break
            print(shot-1)
            nodeId=int(recpos[shot-1][0])
            recZ=int(float(recpos[shot-1][1]))
            recY=int(float(recpos[shot-1][2]))
            recX=int(float(recpos[shot-1][3]))
            curwName="%s/curW_%s.H" % (self.path_out,nodeId)
            oldwName="%s/oldW_%s.H" % (self.path_out,nodeId)
            singleNodeData = "%s/singleNode_%s.H" % (self.path_out,nodeId)
            singlejunk = "%s/junk_%s.H" % (self.path_out,nodeId)
            randomABC = "%s/randomABC_%d.H" % (self.rand_path,nodeId)
            newvel="%s/velwABCs_%d.H" % (self.path_out,nodeId)
            testvel="%s/testvel_%s.H" % (self.path_out,nodeId)
            #
            makenewvel="Add %s %s > %s" % (self.vel_path,randomABC,newvel)
            #
            operator = "%s %s/BORN3d.x" % (self.singularity,self.T_BIN)
            inputs1 = 'par=%s vel=%s rec=%s@@ curW=%s oldW=%s' % (self.genpar,newvel,singlejunk,curwName,oldwName)
            inputs2 = 'souZ=%d souX=%d souY=%d wavelet=%s' % (recZ,recX,recY,self.wavelet)
            header_select = "%s/Window_key synch=1 key1='nodeId' mink1=%s maxk1=%s < %s > %s hff=%s@@" % (self.T_BIN,nodeId,nodeId,self.souhead,singlejunk,singlejunk)
            if self.adjoint==1:

                # # NO MUTING
                # data_select = "cp %s/%s-%s.H %s" % (self.path_out,self.dataname,nodeId,singleNodeData)
                # mute_residuals = " " % ( )

                # WITH MUTING
                data_select = "cp %s/%s-%s.H %s_unmuted" % (self.path_out,self.dataname,nodeId,singleNodeData)
                junk,junk2=commands.getstatusoutput(data_select)
                find_res_header = "Get < %s_unmuted parform=n  hff" % (singleNodeData)
                junk,res_header=commands.getstatusoutput(find_res_header)
                mute_residuals = "%s/NMO_mute.x header=%s offKey=8 < %s_unmuted > %s" % (self.T_BIN,res_header,singleNodeData,singleNodeData)
                
                removeheader = "echo 'hff=-1'>> %s" % (fn)
                outputs = 'adj=1 testvel=%s < %s > %s ; %s' % (testvel, singleNodeData, fn, removeheader)
                headerinit = '  '
                headerwrite = " "
            else:
                # Since the sources are the receiver locations, I call keys 1-3 source positions and 4-6 reciever positions
                header1 = 'echo "hdrkey1=nodeId hdrtype1=scalar_float hdrfmt1=xdr_float" >> %s@@' % (fn)
                header2 = 'echo "hdrkey2=SZ     hdrtype2=scalar_float hdrfmt2=xdr_float" >> %s@@' % (fn)
                header3 = 'echo "hdrkey3=SY     hdrtype3=scalar_float hdrfmt3=xdr_float" >> %s@@' % (fn)
                header4 = 'echo "hdrkey4=SX     hdrtype4=scalar_float hdrfmt4=xdr_float" >> %s@@' % (fn)
                header5 = 'echo "hdrkey5=GZ     hdrtype5=scalar_float hdrfmt5=xdr_float" >> %s@@' % (fn)
                header6 = 'echo "hdrkey6=GY     hdrtype6=scalar_float hdrfmt6=xdr_float" >> %s@@' % (fn)
                header7 = 'echo "hdrkey7=GX     hdrtype7=scalar_float hdrfmt7=xdr_float" >> %s@@' % (fn)
                header8 = 'echo "hdrkey8=OFFSET hdrtype8=scalar_float hdrfmt8=xdr_float" >> %s@@' % (fn)
                headerwrite = "%s; %s; %s; %s; %s; %s; %s; %s;" % (header1, header2, header3, header4, header5, header6, header7, header8)
                data_select = ''
                outputs = 'adj=0 < %s > %s header=%s@@' % (self.modelpath,fn,fn)
                headerinit = 'echo "gff=-1 hff=%s@@" >> %s' % (fn, fn)
            # Assemble final command
            cmd = '%s; %s; %s; %s; %s %s %s %s; %s; %s \n' % (header_select, makenewvel, data_select, mute_residuals, operator, inputs1, inputs2, outputs, headerinit, headerwrite) # Note: need the \n at the end... also cant use < in string itself.
            # Note: need the \n at the end... also cant use < in string itself.
            scripts.append(cmd)

        return scripts, str(subjobid)


    def GetSubjobsInfo(self):
        '''Should return two lists, subjobids and subjobfns.'''
        return copy.deepcopy(self.subjobids), copy.deepcopy(self.subjobfns_list)





class make_BORN(object):
    def __init__(self, param_reader):
        dict_args = param_reader.dict_args
        self.vel_path = dict_args['vel_path']
        self.path_out = dict_args['path_out']
        self.pypath = dict_args['pypath']
        self.T_BIN = dict_args['T_BIN']
        return

    def BORN_run(self, param_reader,fileprefix,cleanup,debug,adjoint,model,dataname):
        #----- RUN BORN OPERATION -------------------
        bornjob = BORN(param_reader, self.vel_path, fileprefix, adjoint, model, dataname, debug)
        bte = BatchTaskExecutor(param_reader)
        if (debug):
            print('-----------------------------------------------------------------')
            print(' RUNNING BORN OPERATOR')
        param_reader.prefix=fileprefix
        bte.LaunchBatchTask(fileprefix, bornjob)

        # Group the output files
        _, fns_list = bornjob.GetSubjobsInfo()
        fn_seph_list = [fn for fns in fns_list for fn in fns]
        args=''
        for i in range(0,len(fn_seph_list)):
            args = '%s %s' % (args,fn_seph_list[i])


        # Cat or Stack depending on the adjoint flag
        if (adjoint):
            cmd = "python %s/recursive_add.py %s %s '%s/%s-*.H | sort -V' " % (self.pypath,self.T_BIN,model,self.path_out,fileprefix)
        # else:
            # cmd = "python %s/recursive_cat.py 30 %s '%s/%s-*.H | sort -V' " % (self.pypath,data,self.path_out,fileprefix)
        if (debug):
            print('-----------------------------------------------------------------')
            print(cmd)
        subprocess.call(cmd,shell=True)


        # # Cat or Stack depending on the adjoint flag
        # if (adjoint):
        #     cmd = 'Add maxsize=10000 %s > %s' % (args,model)
        # else:
        #     cmd = 'Cat3d axis=2 %s hff=%s@@ > %s' % (args,data,data)
        # if (debug):
        #     print('-----------------------------------------------------------------')
        #     print(cmd)
        # subprocess.call(cmd,shell=True)


        # #----- Removing ADJOINT and FORWARD BORN RBF output files from wrk directory -------------
        # if (int(cleanup)==1):
        #     ars_hess=''
        #     for i in range(0,len(fn_seph_list)):
        #       ars_hess = '%s %s' % (ars_hess,fn_seph_list[i])
        #     cmd1 = 'rm -rf %s' % (ars_hess)
        #     cmd2 = 'rm -rf %s/tmp*' % (self.path_out)
        #     cmd3 = 'rm -rf %s/singleres*' % (self.path_out)
        #     cmd  = '%s; %s; %s;' % (cmd1, cmd2, cmd3)
        #     subprocess.call(cmd,shell=True,stdout=False)
        #     print "--------- Removing ADJOINT and FORWARD BORNRBF output files from wrk directory  ------------------------------"
        #     print(cmd)
        return



if __name__ == '__main__':
    print "Run BatchTaskExecutor_test with params:", sys.argv
    # Check basic environment variable setup.
    assert 'SEP' in os.environ, '!Environment var SEP is not set yet, should have your seplib enviroment set up first! Check paths like /opt/SEP/'
    assert 'RSF' in os.environ, '!Environment var RSF is not set yet, should have your RSF environment set up first! Check paths like /opt/RSF'
    eq_args_from_cmdline,args = sepbase.parse_args(sys.argv)
    param_reader = pbs_util.JobParamReader(eq_args_from_cmdline)

    # #------- MAKE THE MODELED DATA ----------------
    # start_time = time.time()
    # datapath = dict_args['datapath']
    # modelpath = dict_args['modelpath']
    # born_obj=make_BORN(param_reader)
    # born_obj.BORN_run(param_reader,True,modelpath,datapath)

    # elapsed_time = (time.time() - start_time)/60.0
    # print("ELAPSED TIME FOR BORN (minutes) : ")
    # print(elapsed_time)
