import subprocess
import numpy as np
import shlex
import os.path
import commands



class MAIN_FUNCTIONS(object):
    def __init__(self, param_reader):
        dict_args = param_reader.dict_args
        self.pathtemp = dict_args['pathtemp']
        self.path_out = dict_args['path_out']
        self.rtm_path= dict_args['rtm_path']
        self.res_path = dict_args['res_path']
        self.T_BIN = dict_args['T_BIN']
        self.vel_path = dict_args['vel_path']
        self.phigrad = dict_args['phigrad']
        self.tomograd = dict_args['tomograd']
        self.genpar = dict_args['genpar']
        self.scaling = dict_args['scaling']

        self.phi_path = dict_args['phi_path']
        self.velback = dict_args['velback']
        self.phi_path = dict_args['phi_path']
        self.rbf_path = dict_args['rbf_path']
        self.hess_out_path = dict_args['hess_out_path']
        self.pypath = dict_args['pypath']

        self.masksalt = dict_args['masksalt']
        self.maskvelb = dict_args['maskvelb']
        self.sparsemodel = dict_args['sparsemodel']
        self.vsalt = dict_args['vsalt']
        self.rbfcoord = dict_args['rbfcoord']
        self.rbftable = dict_args['rbftable']
        self.guide = dict_args['guide']

        self.pad=int(dict_args['pad'])
        self.randpad=int(dict_args['randpad'])

        self.nz=dict_args['nz']
        self.nx=dict_args['nx']
        self.ny=dict_args['ny']

        self.nzr=int(dict_args['nzr'])
        self.nxr=int(dict_args['nxr'])
        self.nyr=int(dict_args['nyr'])

        self.fzr=int(dict_args['f1r'])
        self.fyr=int(dict_args['f2r'])
        self.fxr=int(dict_args['f3r'])

        self.fzR=self.fzr+self.randpad
        self.fxR=self.fxr+self.pad
        self.fyR=self.fyr+self.pad

        self.wavelet = dict_args['wavelet']

        return


    def residual(self,debug,syn_path,obs_path,res_path):
        # #----- MAKE RESIDUAL  ------------------------------------------
        cmd = "Math file1=%s file2=%s exp='(file1-file2)' > %s" % (syn_path,obs_path,res_path) # Need residual to mave meters in d2 and o2 for MUTE_DA to work properly
        if (debug):
            print('-----------------------------------------------------------------')
            print " COMPUTING RESIDUAL "
            print(cmd)
        subprocess.call(cmd,shell=True)
        return


    def MAKE_SCALING(self, debug):
        #------- MAKE SCALING ------------------------------------------
        cmd="Math file1=%s exp='%s-file1'> %s1" % (self.velback,float(self.vsalt),self.scaling)
        cmdwin = "Window3d n1=%d f1=%d n2=%d f2=%d n3=%d f3=%d < %s1 > %s; rm %s1" % (
            self.nzr,self.fzr,self.nxr,self.fxr,self.nyr,self.fyr,self.scaling,self.scaling,self.scaling)
        if (debug):
            print('-----------------------------------------------------------------')
            print('MAKE SCALING')
            print(cmd)
            print(cmdwin)
        subprocess.call(cmd,shell=True)
        subprocess.call(cmdwin,shell=True)
        return


    def COPY(self, debug, fileIN, fileOUT):
        #----- COPY A FILE  -------------------
        cmd1 = "Cp %s %s " % (fileIN,fileOUT)
        aa=commands.getstatusoutput(cmd1)
        if (debug):
            print('-----------------------------------------------------------------')
            print('COPY FILE')
            print(cmd1)
        return


    def HADAMARD_PRODUCT(self, debug, file1, file2, fileOUT):
        #----- HADAMARD_PRODUCT TWO FILES  -------------------
        cmd1 = "Math_base file1=%s file2=%s exp='file1*file2' > %s " % (file1,file2,fileOUT)
        aa=commands.getstatusoutput(cmd1)
        if (debug):
            print('-----------------------------------------------------------------')
            print('HADAMARD_PRODUCT OF TWO FILES (ELEMENT WISE MULTIPLICATION)')
            print(cmd1)
        return



    def WINDOW_RBF(self, debug, fileIN, fileOUT):
        #----- WINDOW A FILE ACCORDING TO THE SMALL RBF AOI WINDOW -------------------
        cmdwin = "Window3d n1=%d f1=%d n2=%d f2=%d n3=%d f3=%d < %s > %s;\n" % (
            self.nzr,self.fzR,self.nxr,self.fxR,self.nyr,self.fyR,fileIN,fileOUT)
        if (debug):
            print('-----------------------------------------------------------------')
            print('WINDOW A FILE ACCORDING TO THE SMALL RBF AOI WINDOW')
            print(cmdwin)
        subprocess.call(cmdwin,shell=True)
        return





    def SCALE(self, debug, fileIN, fileOUT, coeff):
        #----- SCALE A FILE  -------------------
        cmd1 = "Math_base file1=%s exp='file1*%s'> %s " % (fileIN,coeff,fileOUT)
        aa=commands.getstatusoutput(cmd1)
        if (debug):
            print('-----------------------------------------------------------------')
            print('SCALE FILE')
            print(cmd1)
        return



    def MAKE_MASKS_RBF(self, debug, phi_path, masksalt, maskvelb, **optparams):
        if ('smoothrad' in optparams):
            smoothrad = optparams['smoothrad']
        else:
            smoothrad ='5'
        #------- MAKE SALT MASK ------------------------------------------
        cmdwin = "Window3d n1=%d f1=%d n2=%d f2=%d n3=%d f3=%d < %s > %s1" % (
            self.nzr,self.fzr,self.nxr,self.fxr,self.nyr,self.fyr,phi_path,phi_path)
        cmd="python %s/MAKE_MASK.py binpath=%s width=%s guide=%s infile=%s1 masksalt=%s; rm %s1" % (
            self.pypath,self.T_BIN,smoothrad,self.guide,phi_path,masksalt,phi_path)
        if (debug):
            print('-----------------------------------------------------------------')
            print('MAKE SALT MASK')
            print(cmdwin)
            print(cmd)
        subprocess.call(cmdwin,shell=True)
        subprocess.call(cmd,shell=True)
        return


 # For sparsemodel that includes ONLY phi updates
    def DELTAP_BEFORE_RBF1(self, debug, rtm_path, sparsemodel):
        #------- MAKE PHI & TOMO GRADIENTS  ------------------------------------
        cmdwin = "Window3d n1=%d f1=%d n2=%d f2=%d n3=%d f3=%d < %s > %s.trimmed" % (
            self.nzr,self.fzR,self.nxr,self.fxR,self.nyr,self.fyR,rtm_path,rtm_path)
        cmd1 = "%s/APPLY_Dop_RBF3d.x rbftable=%s rbfcoord=%s masksalt=%s scaling=%s verbose=0 adjoint=1 par=%s < %s.trimmed > %s" % (self.T_BIN,self.rbftable,self.rbfcoord,self.masksalt,self.scaling,self.genpar,rtm_path,sparsemodel)
        #------- PURELY FOR VISUALIZATON PURPOSES ------------------------------
        cmd2 = "%s/APPLY_RBF3d.x rbftable=%s rbfcoord=%s verbose=0 adjoint=0 par=%s < %s > %s/fullphigrad.H" % (self.T_BIN,self.rbftable,self.rbfcoord,self.genpar,self.sparsemodel,self.pathtemp)
        cmd = "%s; %s; %s" % (cmdwin,cmd1, cmd2)
        if (debug):
            print('-------------------------------------------------------------')
            print('MAKE PHIrbf GRADIENTS')
            print(cmd)
        subprocess.call(cmd,shell=True)
        return


    def DELTAP_AFTER_RBF1(self, debug, sparseinput, phigrad, tomograd):
        #------- MAKE SCALED RBF GRADIENT TO USE IN LINESEACH  ------------------------------------------
        cmd1 = "Scale < %s > %s_scaled" % (sparseinput, sparseinput)
        #------- CONVERT RBF GRADIENT TO VEL MODEL SPACE FOR VISUALIZING  ------------------------------------------
        cmd2 = "%s/APPLY_RBF3d.x rbftable=%s rbfcoord=%s verbose=0 adjoint=0 par=%s < %s_scaled > %s" % (self.T_BIN,self.rbftable,self.rbfcoord,self.genpar,sparseinput,phigrad)
        cmd3 = "Cp %s %s; Solver_ops op=zero file1=%s" % (phigrad,tomograd,tomograd)
        cmd4 = "rm %s_scaled" % (sparseinput)
        cmd = "%s; %s; %s;" % (cmd1, cmd2, cmd3)
        if (debug):
            print('-----------------------------------------------------------------')
            print('Make phigrad (from RBF model after inversion) and EMPTY tomograd')
            print(cmd)
        subprocess.call(cmd,shell=True)
        return



    def CALC_MAXBETA_RBF(self, debug, upperlim, phigrad, phi_path):
        # --------  Calculate the maxbeta   ----------------------
        operator = '%s/MAXBETA_RBF3d.x upperlim=%s' % (self.T_BIN,upperlim)
        inputs1 = 'phigrad=%s phi=%s par=%s'  % (phigrad,phi_path,self.genpar)
        cmd = '%s %s' % (operator, inputs1)
        if (debug):
            print('-----------------------------------------------------------------')
            print('CALC_MAXBETA')
            print(cmd)
        subprocess.call(cmd,shell=True)
        # --------  Grab the maxbeta   ----------------------
        file = open('maxbetaval.txt')
        for line in file:
            maxbeta = float(line)
        if (debug):
            print('MAXBETA = %s') % (maxbeta)
        return maxbeta



    def SAVE_INTERMEDIATE_FILES(self, debug, invname):
        cmd1 = "%s %s %s %s %s %s %s %sbefore %sbefore %s" % (self.rtm_path,self.vel_path,self.phigrad,self.hess_out_path,self.phi_path,self.tomograd,self.scaling,self.phigrad,self.tomograd,self.rbf_path)
        qq=shlex.split(cmd1)
        if (debug):
            print('-----------------------------------------------------------------')
            print('SAVING INTERMEDIATE FILES')
        for file in qq:
            filecomp = '%scomp' % (file)
            if (os.path.isfile(filecomp)):
                cmd1a = 'Cat %s  %s > %s/out.H' % (filecomp, file, self.path_out)
                cmd1b = 'rm %s ; mv %s/out.H %s' % (filecomp,self.path_out,filecomp)
                cmd= '%s; %s' % (cmd1a, cmd1b)
            else:
                cmd = 'Cp %s %s ' % (file, filecomp)
            cmdp = '%s\n' % (cmd)
            if (debug):
                print(cmdp)
            subprocess.call(cmd,shell=True)

        # Saving the residuals
        filecomp = '%scomp' % (self.res_path)
        if (os.path.isfile(filecomp)):
            cmd1a = 'Cat %s  %s > %s/out.H' % (filecomp, self.res_path, self.path_out)
            cmd1b = 'rm %s ; mv %s/out.H %s' % (filecomp,self.path_out,filecomp)
            cmd= '%s; %s' % (cmd1a, cmd1b)
        else:
            cmd = 'Cp %s %s ' % (self.res_path, filecomp)
        if (debug):
            print(cmd)
        subprocess.call(cmd,shell=True)

        # Saving the hessian inverison stuff
        prefs = "%s %s %s" % ("obj","model","gradient")
        qq=shlex.split(prefs)
        for pref in qq:
            file = '%s%s.H' % (pref,invname)
            filecomp = '%s%s.Hcomp' % (pref,invname)
            if (os.path.isfile(filecomp)):
                cmd1a = 'Cat %s  %s > %s/out.H' % (filecomp, file, self.path_out)
                cmd1b = 'rm %s ; mv %s/out.H %s' % (filecomp,self.path_out,filecomp)
                cmd= '%s; %s' % (cmd1a, cmd1b)
            else:
                cmd = 'Cp %s %s ' % (file, filecomp)
            cmdp = '%s\n' % (cmd)
            if (debug):
                print(cmdp)
            subprocess.call(cmd,shell=True)
        return


    def SAVE_INTERMEDIATE_VALUES(self, debug, objfuncval, maxbeta, final_alpha, final_beta):

        if (debug):
            print('-----------------------------------------------------------------')
            print('SAVING INTERMEDIATE VALUES')

        file1="%s/objfuncval.txt" % (self.pathtemp)
        cmd = "touch %s; echo '%s' >> %s" % (file1,objfuncval,file1)
        if (debug): print(cmd)
        subprocess.call(cmd,shell=True)

        file2="%s/maxbeta.txt" % (self.pathtemp)
        cmd = "touch %s; echo '%s' >> %s" % (file2,maxbeta,file2)
        if (debug): print(cmd)
        subprocess.call(cmd,shell=True)

        file3="%s/final_alpha.txt" % (self.pathtemp)
        cmd = "touch %s; echo '%s' >> %s" % (file3,final_alpha,file3)
        if (debug): print(cmd)
        subprocess.call(cmd,shell=True)

        file4="%s/final_beta.txt" % (self.pathtemp)
        cmd = "touch %s; echo '%s' >> %s" % (file4,final_beta,file4)
        if (debug): print(cmd)
        subprocess.call(cmd,shell=True)
        return


    def CALC_OBJ_VAL(self, debug, objname):
        #----- CALCULATE THE OBJECTIVE FUNCTION VALUE  -------------------
        searchForFiles="ls %s/*%s*.txt" % (self.path_out,objname)
        print(searchForFiles)
        junk,listofResFiles=commands.getstatusoutput(searchForFiles)
        inputfiles = listofResFiles.split()

        # print(inputfiles)

        new_objfuncval=0.0
        for objfile in inputfiles:
            # print(objfile)
            f=open(objfile, "r")
            dotval=float(f.read())
            # dotval=float(aa[1].split()[2])
            new_objfuncval=new_objfuncval+dotval
            print(new_objfuncval)

        return new_objfuncval



    def CLEAN_UP(self, debug, invname):
        #----- Write the velmodel to output (current iteration) --------------------
        cmd1 = 'rm -f %s/*' % (self.path_out)
        cmd2 = 'rm -f %s' % (self.hess_out_path)
        cmd3 = 'rm -f %s' % (invname)
        cmd =  "%s; %s; %s;" % (cmd1, cmd2, cmd3)
        if (debug):
            print('-----------------------------------------------------------------')
            print('Remove WRK and Inversion output')
            print(cmd)
        subprocess.call(cmd,shell=True)
        return


    def RM_SRC_WAVELET(self, debug):
        #----- Remove the source wavelet --------------------
        cmd = 'rm -f %s' % (self.wavelet)
        if (debug):
            print('-----------------------------------------------------------------')
            print('Remove SOURCE WAVELET')
            print(cmd)
        subprocess.call(cmd,shell=True)
        return


    def CLEAN_TMP(self, debug):
        #----- Write the velmodel to output (current iteration) --------------------
        cmd = 'rm %s/*' % (self.path_tmp)
        if (debug):
            print('-----------------------------------------------------------------')
            print('CLEAN_TMP')
            print(cmd)
        subprocess.call(cmd,shell=True)
        return
